@@ -1,5 +1,5 @@
 """
-Simple minimizer  is a wrapper around scipy.leastsq, allowing a
+Simple minimizer is a wrapper around scipy.leastsq, allowing a
 user to build a fitting model as a function of general purpose
 Fit Parameters that can be fixed or floated, bounded, and written
 as a simple expression of other Fit Parameters.
@@ -14,7 +14,7 @@ function-to-be-minimized (residual function) in terms of these Parameters.
 from copy import deepcopy
 import numpy as np
 from numpy import (dot, eye, ndarray, ones_like,
-                   sqrt, take, transpose, triu)
+                   sqrt, take, transpose, triu, deprecate)
 from numpy.dual import inv
 from numpy.linalg import LinAlgError
 
@@ -22,6 +22,12 @@ from scipy.optimize import leastsq as scipy_leastsq
 from scipy.optimize import fmin as scipy_fmin
 from scipy.optimize.lbfgsb import fmin_l_bfgs_b as scipy_lbfgsb
 
+# differential_evolution is only present in scipy >= 0.15
+try:
+    from scipy.optimize import differential_evolution as scipy_diffev
+except ImportError:
+    from ._differentialevolution import differential_evolution as scipy_diffev
+
 # check for scipy.optimize.minimize
 HAS_SCALAR_MIN = False
 try:
@@ -38,7 +44,7 @@ from .parameter import Parameter, Parameters
 from . import uncertainties
 
 
-def asteval_with_uncertainties(*vals,  **kwargs):
+def asteval_with_uncertainties(*vals, **kwargs):
     """
     given values for variables, calculate object value.
     This is used by the uncertainties package to calculate
@@ -61,7 +67,6 @@ def asteval_with_uncertainties(*vals,  **kwargs):
 
 wrap_ueval = uncertainties.wrap(asteval_with_uncertainties)
 
-
 def eval_stderr(obj, uvars, _names, _pars, _asteval):
     """evaluate uncertainty and set .stderr for a parameter `obj`
     given the uncertain values `uvars` (a list of uncertainties.ufloats),
@@ -99,6 +104,21 @@ def check_ast_errors(error):
             return msg
     return None
 
+def _differential_evolution(func, x0, **kwds):
+    """
+    A wrapper for differential_evolution that can be used with scipy.minimize
+    """
+    kwargs = dict(args=(), strategy='best1bin', maxiter=None, popsize=15,
+                  tol=0.01, mutation=(0.5, 1), recombination=0.7, seed=None,
+                  callback=None, disp=False, polish=True,
+                  init='latinhypercube')
+
+    for k, v in kwds.items():
+        if k in kwargs:
+            kwargs[k] = v
+
+    return scipy_diffev(func, kwds['bounds'], **kwargs)
+
 SCALAR_METHODS = {'nelder': 'Nelder-Mead',
                   'powell': 'Powell',
                   'cg': 'CG',
@@ -110,18 +130,64 @@ SCALAR_METHODS = {'nelder': 'Nelder-Mead',
                   'cobyla': 'COBYLA',
                   'slsqp': 'SLSQP',
                   'dogleg': 'dogleg',
-                  'trust-ncg': 'trust-ncg'}
+                  'trust-ncg': 'trust-ncg',
+                  'differential_evolution': 'differential_evolution'}
+
 
 class Minimizer(object):
-    """general minimizer"""
-    err_nonparam = \
-     "params must be a minimizer.Parameters() instance or list of Parameters()"
-    err_maxfev = """Too many function calls (max set to  %i)!  Use:
-    minimize(func, params, ...., maxfev=NNN)
-or set  leastsq_kws['maxfev']  to increase this maximum."""
+    """A general minimizer for curve fitting"""
+    err_nonparam = ("params must be a minimizer.Parameters() instance or list "
+                    "of Parameters()")
+    err_maxfev = ("Too many function calls (max set to %i)!  Use:"
+                  " minimize(func, params, ..., maxfev=NNN)"
+                  "or set leastsq_kws['maxfev']  to increase this maximum.")
 
     def __init__(self, userfcn, params, fcn_args=None, fcn_kws=None,
                  iter_cb=None, scale_covar=True, **kws):
+        """
+        Initialization of the Minimzer class
+
+        Parameters
+        ----------
+        userfcn : callable
+            objective function that returns the residual (difference between
+            model and data) to be minimized in a least squares sense.  The
+            function must have the signature:
+            `userfcn(params, *fcn_args, **fcn_kws)`
+        params : lmfit.parameter.Parameters object.
+            contains the Parameters for the model.
+        fcn_args : tuple, optional
+            positional arguments to pass to userfcn.
+        fcn_kws : dict, optional
+            keyword arguments to pass to userfcn.
+        iter_cb : callable, optional
+            Function to be called at each fit iteration. This function should
+            have the signature:
+            `iter_cb(params, iter, resid, *fcn_args, **fcn_kws)`,
+            where where `params` will have the current parameter values, `iter`
+            the iteration, `resid` the current residual array, and `*fcn_args`
+            and `**fcn_kws` as passed to the objective function.
+        scale_covar : bool, optional
+            Whether to automatically scale the covariance matrix (leastsq
+            only).
+        kws : dict, optional
+            Options to pass to the minimizer being used.
+
+        Notes
+        -----
+        The objective function should return the value to be minimized. For the
+        Levenberg-Marquardt algorithm from leastsq(), this returned value must
+        be an array, with a length greater than or equal to the number of
+        fitting variables in the model. For the other methods, the return value
+        can either be a scalar or an array. If an array is returned, the sum of
+        squares of the array will be sent to the underlying fitting method,
+        effectively doing a least-squares optimization of the return values.
+
+        A common use for the fcn_args and fcn_kwds would be to pass in other
+        data needed to calculate the residual, including such things as the
+        data array, dependent variable, uncertainties in the data, and other
+        data structures for the model calculation.
+        """
         self.userfcn = userfcn
         self.userargs = fcn_args
         if self.userargs is None:
@@ -138,13 +204,13 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.ndata = 0
         self.nvarys = 0
         self.ier = 0
-        self.success = None
+        self.success = True
+        self.errorbars = False
         self.message = None
         self.lmdif_message = None
         self.chisqr = None
         self.redchi = None
         self.covar = None
-        self.errorbars = None
         self.residual = None
         self.var_map = []
         self.vars = []
@@ -159,7 +225,13 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
     @property
     def values(self):
-        "Convenience function that returns Parameter values as a simple dict."
+        """
+        Returns
+        -------
+        param_values : dict
+            Parameter values in a simple dictionary.
+        """
+
         return dict([(name, p.value) for name, p in self.params.items()])
 
     def __update_paramval(self, name):
@@ -189,19 +261,21 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.updated[name] = True
 
     def update_constraints(self):
-        """update all constrained parameters, checking that
-        dependencies are evaluated as needed."""
+        """
+        Update all constrained parameters, checking that dependencies are
+        evaluated as needed.
+        """
         self.updated = dict([(name, False) for name in self.params])
         for name in self.params:
             self.__update_paramval(name)
 
     def __residual(self, fvars):
         """
-        residual function used for least-squares fit.
-        With the new, candidate values of fvars (the fitting variables),
-        this evaluates all parameters, including setting bounds and
-        evaluating constraints, and then passes those to the
-        user-supplied function to calculate the residual.
+        Residual function used for least-squares fit.
+        With the new, candidate values of fvars (the fitting variables), this
+        evaluates all parameters, including setting bounds and evaluating
+        constraints, and then passes those to the user-supplied function to
+        calculate the residual.
         """
         # set parameter values
         for varname, val in zip(self.var_map, fvars):
@@ -212,7 +286,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         self.update_constraints()
         out = self.userfcn(self.params, *self.userargs, **self.userkws)
-        if hasattr(self.iter_cb, '__call__'):
+        if callable(self.iter_cb):
             self.iter_cb(self.params, self.nfev, out,
                          *self.userargs, **self.userkws)
         return out
@@ -249,9 +323,19 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             raise MinimizerException(self.err_nonparam)
 
     def penalty(self, params):
-        """penalty function for scalar minimizers:
-        evaluates user-supplied objective function,
-        if result is an array, return array sum-of-squares.
+        """
+        Penalty function for scalar minimizers:
+
+        Parameters
+        ----------
+        params : lmfit.parameter.Parameters object
+            The model parameters
+
+        Returns
+        -------
+        r - float
+            The user evaluated user-supplied objective function. If the
+            objective function is an array, return the array sum-of-squares
         """
         r = self.__residual(params)
         if isinstance(r, ndarray):
@@ -259,7 +343,9 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         return r
 
     def prepare_fit(self, params=None):
-        """prepare parameters for fit"""
+        """
+        Prepares parameters for fitting
+        """
         # determine which parameters are actually variables
         # and which are defined expressions.
         if params is None and self.params is not None and self.__prepared:
@@ -299,8 +385,9 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.__prepared = True
 
     def unprepare_fit(self):
-        """unprepare fit, so that subsequent fits will be
-        forced to run re-prepare the fit
+        """
+        Unprepares the fit, so that subsequent fits will be
+        forced to run prepare_fit.
 
         removes ast compilations of constraint expressions
         """
@@ -310,10 +397,20 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             if hasattr(par, 'ast'):
                 delattr(par, 'ast')
 
+    @deprecate(message='    Deprecated in lmfit 0.8.2, use scalar_minimize '
+                       'and method=\'L-BFGS-B\' instead')
     def lbfgsb(self, **kws):
         """
-        use l-bfgs-b minimization
+        Use l-bfgs-b minimization
+
+        Parameters
+        ----------
+        kws : dict
+            Minimizer options to pass to the
+            scipy.optimize.lbfgsb.fmin_l_bfgs_b function.
+
         """
+
         self.prepare_fit()
         lb_kws = dict(factr=1000.0, approx_grad=True, m=20,
                       maxfun=2000 * (self.nvarys + 1),
@@ -335,10 +432,18 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.unprepare_fit()
         return
 
+    @deprecate(message='    Deprecated in lmfit 0.8.2, use scalar_minimize '
+                       'and method=\'Nelder-Mead\' instead')
     def fmin(self, **kws):
         """
-        use nelder-mead (simplex) minimization
+        Use Nelder-Mead (simplex) minimization
+
+        Parameters
+        ----------
+        kws : dict
+            Minimizer options to pass to the scipy.optimize.fmin minimizer.
         """
+
         self.prepare_fit()
         fmin_kws = dict(full_output=True, disp=False, retall=True,
                         ftol=1.e-4, xtol=1.e-4,
@@ -361,19 +466,29 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         return
 
     def scalar_minimize(self, method='Nelder-Mead', **kws):
-        """use one of the scaler minimization methods from scipy.
-        Available methods include:
-          Nelder-Mead
-          Powell
-          CG  (conjugate gradient)
-          BFGS
-          Newton-CG
-          L-BFGS-B
-          TNC
-          COBYLA
-          SLSQP
-          dogleg
-          trust-ncg
+        """
+        Use one of the scalar minimization methods from
+        scipy.optimize.minimize.
+
+        Parameters
+        ----------
+        method : str, optional
+            Name of the fitting method to use.
+            One of:
+                'Nelder-Mead' (default)
+                'L-BFGS-B'
+                'Powell'
+                'CG'
+                'Newton-CG'
+                'COBYLA'
+                'TNC'
+                'trust-ncg'
+                'dogleg'
+                'SLSQP'
+                'differential_evolution'
+
+        kws : dict, optional
+            Minimizer options pass to scipy.optimize.minimize.
 
         If the objective function returns a numpy array instead
         of the expected scalar, the sum of squares of the array
@@ -381,16 +496,22 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         Note that bounds and constraints can be set on Parameters
         for any of these methods, so are not supported separately
-        for those designed to use bounds.
+        for those designed to use bounds. However, if you use the
+        differential_evolution option you must specify finite
+        (min, max) for each Parameter.
+
+        Returns
+        -------
+        success : bool
+            Whether the fit was successful.
 
         """
         if not HAS_SCALAR_MIN:
             raise NotImplementedError
-
         self.prepare_fit()
 
         fmin_kws = dict(method=method,
-                        options={'maxiter': 1000*(self.nvarys + 1)})
+                        options={'maxiter': 1000 * (self.nvarys + 1)})
         fmin_kws.update(self.kws)
         fmin_kws.update(kws)
 
@@ -409,9 +530,20 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             self.jacfcn = None
             fmin_kws.pop('jac')
 
+        if method == 'differential_evolution':
+            fmin_kws['method'] = _differential_evolution
+            pars = self.params
+            bounds = [(pars[par].min, pars[par].max) for par in pars]
+            if not np.all(np.isfinite(bounds)):
+                raise ValueError('With differential evolution finite bounds '
+                                 'are required for each parameter')
+            bounds = [(-np.pi / 2., np.pi / 2.)] * len(self.vars)
+            fmin_kws['bounds'] = bounds
+
         ret = scipy_minimize(self.penalty, self.vars, **fmin_kws)
         xout = ret.x
         self.message = ret.message
+
         self.nfev = ret.nfev
         self.chisqr = self.residual = self.__residual(xout)
         self.ndata = 1
@@ -420,24 +552,32 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             self.chisqr = (self.chisqr**2).sum()
             self.ndata = len(self.residual)
             self.nfree = self.ndata - self.nvarys
-        self.redchi = self.chisqr/self.nfree
+        self.redchi = self.chisqr / self.nfree
         self.unprepare_fit()
-        return
+        return ret.success
 
     def leastsq(self, **kws):
         """
-        use Levenberg-Marquardt minimization to perform fit.
-        This assumes that ModelParameters have been stored,
-        and a function to minimize has been properly set up.
+        Use Levenberg-Marquardt minimization to perform a fit.
+        This assumes that ModelParameters have been stored, and a function to
+        minimize has been properly set up.
 
-        This wraps scipy.optimize.leastsq, and keyword arguments are passed
-        directly as options to scipy.optimize.leastsq
+        This wraps scipy.optimize.leastsq.
 
         When possible, this calculates the estimated uncertainties and
         variable correlations from the covariance matrix.
 
-        writes outputs to many internal attributes, and
-        returns True if fit was successful, False if not.
+        Writes outputs to many internal attributes.
+
+        Parameters
+        ----------
+        kws : dict, optional
+            Minimizer options to pass to scipy.optimize.leastsq.
+
+        Returns
+        -------
+        success : bool
+            True if fit was successful, False if not.
         """
         self.prepare_fit()
         lskws = dict(full_output=1, xtol=1.e-7, ftol=1.e-7,
@@ -559,7 +699,33 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         return self.success
 
     def minimize(self, method='leastsq'):
-        """perform minimization using supplied method"""
+        """
+        Perform the minimization.
+
+        Parameters
+        ----------
+        method : str, optional
+            Name of the fitting method to use.
+            One of:
+            'leastsq'                -    Levenberg-Marquardt (default)
+            'nelder'                 -    Nelder-Mead
+            'lbfgsb'                 -    L-BFGS-B
+            'powell'                 -    Powell
+            'cg'                     -    Conjugate-Gradient
+            'newton'                 -    Newton-CG
+            'cobyla'                 -    Cobyla
+            'tnc'                    -    Truncate Newton
+            'trust-ncg'              -    Trust Newton-CGn
+            'dogleg'                 -    Dogleg
+            'slsqp'                  -    Sequential Linear Squares Programming
+            'differential_evolution' -    differential evolution
+
+        Returns
+        -------
+        success : bool
+            Whether the fit was successful.
+        """
+
         function = self.leastsq
         kwargs = {}
 
@@ -571,7 +737,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             for key, val in SCALAR_METHODS.items():
                 if (key.lower().startswith(user_method) or
                     val.lower().startswith(user_method)):
-                    kwargs = dict(method=val)
+                    kwargs['method'] = val
         elif (user_method.startswith('nelder') or
               user_method.startswith('fmin')):
             function = self.fmin
@@ -583,9 +749,67 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
 def minimize(fcn, params, method='leastsq', args=None, kws=None,
              scale_covar=True, iter_cb=None, **fit_kws):
-    """simple minimization function,
-    finding the values for the params which give the
-    minimal sum-of-squares of the array return by fcn
+    """
+    A general purpose curvefitting function
+    The minimize function takes a objective function to be minimized, a
+    dictionary (lmfit.parameter.Parameters) containing the model parameters,
+    and several optional arguments.
+
+    Parameters
+    ----------
+    fcn : callable
+        objective function that returns the residual (difference between
+        model and data) to be minimized in a least squares sense.  The
+        function must have the signature:
+        `fcn(params, *args, **kws)`
+    params : lmfit.parameter.Parameters object.
+        contains the Parameters for the model.
+    method : str, optional
+        Name of the fitting method to use.
+        One of:
+            'leastsq'                -    Levenberg-Marquardt (default)
+            'nelder'                 -    Nelder-Mead
+            'lbfgsb'                 -    L-BFGS-B
+            'powell'                 -    Powell
+            'cg'                     -    Conjugate-Gradient
+            'newton'                 -    Newton-CG
+            'cobyla'                 -    Cobyla
+            'tnc'                    -    Truncate Newton
+            'trust-ncg'              -    Trust Newton-CGn
+            'dogleg'                 -    Dogleg
+            'slsqp'                  -    Sequential Linear Squares Programming
+            'differential_evolution' -    differential evolution
+
+    args : tuple, optional
+        Positional arguments to pass to fcn.
+    kws : dict, optional
+        keyword arguments to pass to fcn.
+    iter_cb : callable, optional
+        Function to be called at each fit iteration. This function should
+        have the signature `iter_cb(params, iter, resid, *args, **kws)`,
+        where where `params` will have the current parameter values, `iter`
+        the iteration, `resid` the current residual array, and `*args`
+        and `**kws` as passed to the objective function.
+    scale_covar : bool, optional
+        Whether to automatically scale the covariance matrix (leastsq
+        only).
+    fit_kws : dict, optional
+        Options to pass to the minimizer being used.
+
+    Notes
+    -----
+    The objective function should return the value to be minimized. For the
+    Levenberg-Marquardt algorithm from leastsq(), this returned value must
+    be an array, with a length greater than or equal to the number of
+    fitting variables in the model. For the other methods, the return value
+    can either be a scalar or an array. If an array is returned, the sum of
+    squares of the array will be sent to the underlying fitting method,
+    effectively doing a least-squares optimization of the return values.
+
+    A common use for `args` and `kwds` would be to pass in other
+    data needed to calculate the residual, including such things as the
+    data array, dependent variable, uncertainties in the data, and other
+    data structures for the model calculation.
     """
     fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
                        iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
