@@ -4,11 +4,18 @@ Concise nonlinear curve fitting.
 from __future__ import print_function
 import warnings
 import inspect
+import operator
 from copy import deepcopy
 import numpy as np
 from . import Parameters, Parameter, Minimizer
 from .printfuncs import fit_report
 
+try:
+    from collections import OrderedDict
+except ImportError:
+    from ordereddict import OrderedDict
+
+
 # Use pandas.isnull for aligning missing data is pandas is available.
 # otherwise use numpy.isnan
 try:
@@ -25,6 +32,24 @@ def _align(var, mask, data):
         return var[mask]
     return var
 
+
+try:
+    from matplotlib import pyplot as plt
+    _HAS_MATPLOTLIB = True
+except ImportError:
+    _HAS_MATPLOTLIB = False
+
+
+def _ensureMatplotlib(function):
+    if _HAS_MATPLOTLIB:
+        return function
+    else:
+        def no_op(*args, **kwargs):
+            print('matplotlib module is required for plotting the results')
+
+        return no_op
+
+
 class Model(object):
     """Create a model from a user-defined function.
 
@@ -64,7 +89,6 @@ class Model(object):
     _invalid_par   = "Invalid parameter name ('%s') for function %s"
     _invalid_missing = "missing must be None, 'none', 'drop', or 'raise'."
     _valid_missing   = (None, 'none', 'drop', 'raise')
-    _names_collide = "Two models have parameters named %s. Use distinct names"
 
     _invalid_hint = "unknown parameter hint '%s' for param '%s'"
     _hint_names = ('value', 'vary', 'min', 'max', 'expr')
@@ -75,7 +99,6 @@ class Model(object):
         self._prefix = prefix
         self._param_root_names = param_names  # will not include prefixes
         self.independent_vars = independent_vars
-        self.components = []
         self._func_allargs = []
         self._func_haskeywords = False
         if not missing in self._valid_missing:
@@ -92,29 +115,20 @@ class Model(object):
         self._name = name
 
     def _reprstring(self, long=False):
-        if not self.is_composite:
-            # base model
-            opts = []
-            if len(self._prefix) > 0:
-                opts.append("prefix='%s'" % (self._prefix))
-            if long:
-                for k, v in self.opts.items():
-                    opts.append("%s='%s'" % (k, v))
-
-            out = ["%s" % self._name]
-            if len(opts) > 0:
-                out[0] = "%s(%s)" % (out[0], ','.join(opts))
-        else:
-            # composite model
-            if self._name is None:
-                out = [c._reprstring(long)[0] for c in self.components]
-            else:
-                out = [self._name]
-        return out
+        out = self._name
+        opts = []
+        if len(self._prefix) > 0:
+            opts.append("prefix='%s'" % (self._prefix))
+        if long:
+            for k, v in self.opts.items():
+                opts.append("%s='%s'" % (k, v))
+        if len(opts) > 0:
+            out = "%s, %s" % (out, ', '.join(opts))
+        return "Model(%s)" % out
 
     @property
     def name(self):
-        return '+'.join(self._reprstring(long=False))
+        return self._reprstring(long=False)
 
     @name.setter
     def name(self, value):
@@ -131,21 +145,7 @@ class Model(object):
 
     @property
     def param_names(self):
-        if self.is_composite:
-            return self._compute_composite_param_names()
-        else:
-            return self._param_names
-
-    def _compute_composite_param_names(self):
-        param_names = set()
-        for sub_model in self.components:
-            param_names |= sub_model.param_names
-        param_names |= self._param_names
-        return param_names
-
-    @property
-    def is_composite(self):
-        return len(self.components) > 0
+        return self._param_names
 
     def __repr__(self):
         return  "<lmfit.Model: %s>" % (self.name)
@@ -263,45 +263,27 @@ class Model(object):
         if 'verbose' in kwargs:
             verbose = kwargs['verbose']
         params = Parameters()
-        if not self.is_composite:
-            # base model: build Parameters from scratch
-            for name in self.param_names:
-                par = Parameter(name=name)
-                basename = name[len(self._prefix):]
-                # apply defaults from model function definition
-                if basename in self.def_vals:
-                    par.value = self.def_vals[basename]
-                # apply defaults from parameter hints
-                if basename in self.param_hints:
-                    hint = self.param_hints[basename]
-                    for item in self._hint_names:
-                        if item in  hint:
-                            setattr(par, item, hint[item])
-                # apply values passed in through kw args
-                if basename in kwargs:
-                    # kw parameter names with no prefix
-                    par.value = kwargs[basename]
-                if name in kwargs:
-                    # kw parameter names with prefix
-                    par.value = kwargs[name]
-                params[name] = par
-        else:
-            # composite model: merge the sub_models parameters adding hints
-            for sub_model in self.components:
-                comp_params = sub_model.make_params(**kwargs)
-                for par_name, param in comp_params.items():
-                    # apply composite-model hints
-                    if par_name in self.param_hints:
-                        hint = self.param_hints[par_name]
-                        for item in self._hint_names:
-                            if item in  hint:
-                                setattr(param, item, hint[item])
-                params.update(comp_params)
 
-            # apply defaults passed in through kw args
-            for name in self.param_names:
-                if name in kwargs:
-                    params[name].value = kwargs[name]
+        for name in self.param_names:
+            par = Parameter(name=name)
+            basename = name[len(self._prefix):]
+            # apply defaults from model function definition
+            if basename in self.def_vals:
+                par.value = self.def_vals[basename]
+            # apply defaults from parameter hints
+            if basename in self.param_hints:
+                hint = self.param_hints[basename]
+                for item in self._hint_names:
+                    if item in  hint:
+                        setattr(par, item, hint[item])
+            # apply values passed in through kw args
+            if basename in kwargs:
+                # kw parameter names with no prefix
+                par.value = kwargs[basename]
+            if name in kwargs:
+                # kw parameter names with prefix
+                par.value = kwargs[name]
+            params[name] = par
 
         # add any additional parameters defined in param_hints
         # note that composites may define their own additional
@@ -379,25 +361,32 @@ class Model(object):
         args = {}
         for key, val in self.make_funcargs(params, kwargs).items():
             args["%s%s" % (self._prefix, key)] = val
-        for sub_model in self.components:
-            otherargs = sub_model._make_all_args(params, **kwargs)
-            args.update(otherargs)
         return args
 
     def eval(self, params=None, **kwargs):
         """evaluate the model with the supplied parameters"""
-        if len(self.components) > 0:
-            result = self.components[0].eval(params, **kwargs)
-            for model in self.components[1:]:
-                result += model.eval(params, **kwargs)
-        else:
-            result = self.func(**self.make_funcargs(params, kwargs))
-            # Handle special case of constant result and one
-            # independent variable (of any dimension).
-            if np.ndim(result) == 0 and len(self.independent_vars) == 1:
-                result = np.tile(result, kwargs[self.independent_vars[0]].shape)
+        result = self.func(**self.make_funcargs(params, kwargs))
+        # Handle special case of constant result and one
+        # independent variable (of any dimension).
+        if np.ndim(result) == 0 and len(self.independent_vars) == 1:
+            result = np.tile(result, kwargs[self.independent_vars[0]].shape)
         return result
 
+    @property
+    def components(self):
+        """return components for composite model"""
+        return [self]
+
+    def eval_components(self, params=None, **kwargs):
+        """
+        evaluate the model with the supplied parameters and returns a ordered
+        dict containting name, result pairs.
+        """
+        key = self._prefix
+        if len(key) < 1:
+            key = self._name
+        return {key: self.eval(params=params, **kwargs)}
+
     def fit(self, data, params=None, weights=None, method='leastsq',
             iter_cb=None, scale_covar=True, verbose=True, fit_kws=None, **kwargs):
         """Fit the model to the data.
@@ -504,29 +493,113 @@ class Model(object):
         output = ModelFit(self, params, method=method, iter_cb=iter_cb,
                           scale_covar=scale_covar, fcn_kws=kwargs, **fit_kws)
         output.fit(data=data, weights=weights)
+        output.components = self.components
         return output
 
     def __add__(self, other):
-        colliding_param_names = self.param_names & other.param_names
-        if len(colliding_param_names) != 0:
-            collision = colliding_param_names.pop()
-            raise NameError(self._names_collide % collision)
+        return CompositeModel(self, other, operator.add)
 
-        # If the model is already composite just add other as component
-        composite = self
-        if not self.is_composite:
-            # make new composite Model, add self and other as components
-            composite = Model(func=None)
-            composite.components = [self]
-            # we assume that all the sub-models have the same independent vars
-            composite.independent_vars = self.independent_vars[:]
+    def __sub__(self, other):
+        return CompositeModel(self, other, operator.sub)
 
-        if other.is_composite:
-            composite.components.extend(other.components)
-            composite.param_hints.update(other.param_hints)
-        else:
-            composite.components.append(other)
-        return composite
+    def __mul__(self, other):
+        return CompositeModel(self, other, operator.mul)
+
+    def __div__(self, other):
+        return CompositeModel(self, other, operator.truediv)
+
+    def __truediv__(self, other):
+        return CompositeModel(self, other, operator.truediv)
+
+
+class CompositeModel(Model):
+    """Create a composite model -- a binary operator of two Models
+
+    Parameters
+    ----------
+    left_model:    left-hand side model-- must be a Model()
+    right_model:   right-hand side model -- must be a Model()
+    oper:          callable binary operator (typically, operator.add, operator.mul, etc)
+
+    independent_vars: list of strings or None (default)
+        arguments to func that are independent variables
+    param_names: list of strings or None (default)
+        names of arguments to func that are to be made into parameters
+    missing: None, 'none', 'drop', or 'raise'
+        'none' or None: Do not check for null or missing values (default)
+        'drop': Drop null or missing observations in data.
+            if pandas is installed, pandas.isnull is used, otherwise
+            numpy.isnan is used.
+        'raise': Raise a (more helpful) exception when data contains null
+            or missing values.
+    name: None or string
+        name for the model. When `None` (default) the name is the same as
+        the model function (`func`).
+
+    """
+    _names_collide = "Two models have parameters named {clash}. Use distinct names"
+    _bad_arg   = "CompositeModel: argument {arg} is not a Model"
+    _bad_op    = "CompositeModel: operator {op} is not callable"
+    _known_ops = {operator.add: '+', operator.sub: '-',
+                  operator.mul: '*', operator.truediv: '/'}
+
+    def __init__(self, left, right, op, **kws):
+        if not isinstance(left, Model):
+            raise ValueError(self._bad_arg.format(arg=left))
+        if not isinstance(right, Model):
+            raise ValueError(self._bad_arg.format(arg=right))
+        if not callable(op):
+            raise ValueError(self._bad_op.format(op=op))
+
+        self.left  = left
+        self.right = right
+        self.op    = op
+
+        name_collisions = left.param_names & right.param_names
+        if len(name_collisions) > 0:
+            raise NameError(self._names_collide.format(clash=name_collisions))
+
+        if 'independent_vars' not in kws:
+            kws['independent_vars'] = self.left.independent_vars
+        if 'missing' not in kws:
+            kws['missing'] = self.left.missing
+
+        def _tmp(self, *args, **kws): pass
+        Model.__init__(self, _tmp, **kws)
+
+    def _parse_params(self):
+        self._func_haskeywords = (self.left._func_haskeywords or
+                                  self.right._func_haskeywords)
+        self._func_allargs = (self.left._func_allargs +
+                              self.right._func_allargs)
+        self.def_vals = deepcopy(self.right.def_vals)
+        self.def_vals.update(self.left.def_vals)
+        self.opts = deepcopy(self.right.opts)
+        self.opts.update(self.left.opts)
+
+    def _reprstring(self, long=False):
+        return "(%s %s %s)" % (self.left._reprstring(long=long),
+                               self._known_ops.get(self.op, self.op),
+                               self.right._reprstring(long=long))
+
+    def eval(self, params=None, **kwargs):
+        return self.op(self.left.eval(params=params, **kwargs),
+                       self.right.eval(params=params, **kwargs))
+
+    def eval_components(self, **kwargs):
+        """return ordered dict of name, results for each component"""
+        out = OrderedDict(self.left.eval_components(**kwargs))
+        out.update(self.right.eval_components(**kwargs))
+        return out
+
+    @property
+    def param_names(self):
+        return self.left.param_names | self.right.param_names
+
+    @property
+    def components(self):
+        """return components for composite model"""
+        return self.left.components + self.right.components
 
 
 class ModelFit(Minimizer):
@@ -558,9 +631,28 @@ class ModelFit(Minimizer):
          evaluate the current model, with the current parameter values,
          with values in kwargs sent to the model function.
 
+    eval_components(**kwargs)
+         evaluate the current model, with the current parameter values,
+         with values in kwargs sent to the model function and returns
+         a ordered dict with the model names as the key and the component
+         results as the values.
+
    fit_report(modelpars=None, show_correl=True, min_correl=0.1)
          return a fit report.
 
+   plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--',
+            numpoints=None,  data_kws=None, fit_kws=None, init_kws=None,
+            ax_kws=None)
+        Plot the fit results using matplotlib.
+
+   plot_residuals(self, ax=None, datafmt='o', data_kws=None, fit_kws=None,
+                  ax_kws=None)
+        Plot the fit residuals using matplotlib.
+
+   plot(self, datafmt='o', fitfmt='-', initfmt='--', numpoints=None,
+        data_kws=None, fit_kws=None, init_kws=None, ax_res_kws=None,
+        ax_fit_kws=None, fig_kws=None)
+        Plot the fit results and residuals using matplotlib.
     """
     def __init__(self, model, params, data=None, weights=None,
                  method='leastsq', fcn_args=None, fcn_kws=None,
@@ -598,18 +690,285 @@ class ModelFit(Minimizer):
         self.userkws.update(kwargs)
         return self.model.eval(params=self.params, **self.userkws)
 
+    def eval_components(self, **kwargs):
+        self.userkws.update(kwargs)
+        return self.model.eval_components(params=self.params, **self.userkws)
+
     def fit_report(self, modelpars=None, show_correl=True, min_correl=0.1):
         "return fit report"
         stats_report = fit_report(self, modelpars=modelpars,
                                  show_correl=show_correl,
                                  min_correl=min_correl)
         buff = ['[[Model]]']
-        if len(self.model.components)==0:
-            buff.append('    %s' % self.model._reprstring(long=True)[0])
-        else:
-            buff.append(' Composite Model:')
-            for x in self.model._reprstring(long=True):
-                buff.append('    %s' % x)
+        buff.append('    %s' % self.model._reprstring(long=True))
         buff = '\n'.join(buff)
         out = '%s\n%s' % (buff, stats_report)
         return out
+
+    @_ensureMatplotlib
+    def plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--',
+                 numpoints=None,  data_kws=None, fit_kws=None, init_kws=None,
+                 ax_kws=None):
+        """Plot the fit results using matplotlib.
+
+        The method will plot results of the fit using matplotlib, including:
+        the data points, the initial fit curve and the fitted curve. If the fit
+        model included weights, errorbars will also be plotted.
+
+        Parameters
+        ----------
+        ax : matplotlib.axes.Axes, optional
+            The axes to plot on. The default in None, which means use the
+            current pyplot axis or create one if there is none.
+        datafmt : string, optional
+            matplotlib format string for data points
+        fitfmt : string, optional
+            matplotlib format string for fitted curve
+        initfmt : string, optional
+            matplotlib format string for initial conditions for the fit
+        numpoints : int, optional
+            If provided, the final and initial fit curves are evaluated not
+            only at data points, but refined to contain `numpoints` points in
+            total.
+        data_kws : dictionary, optional
+            keyword arguments passed on to the plot function for data points
+        fit_kws : dictionary, optional
+            keyword arguments passed on to the plot function for fitted curve
+        init_kws : dictionary, optional
+            keyword arguments passed on to the plot function for the initial
+            conditions of the fit
+        ax_kws : dictionary, optional
+            keyword arguments for a new axis, if there is one being created
+
+        Returns
+        -------
+        matplotlib.axes.Axes
+
+        Notes
+        ----
+        For details about plot format strings and keyword arguments see
+        documentation of matplotlib.axes.Axes.plot.
+
+        If the fit model included weights, then matplotlib.axes.Axes.errorbar
+        is used to plot the data, with yerr set to 1 / self.weights**0.5
+
+        If `ax` is None then matplotlib.pyplot.gca(**ax_kws) is called.
+
+        See Also
+        --------
+        ModelFit.plot_residuals : Plot the fit residuals using matplotlib.
+        ModelFit.plot : Plot the fit results and residuals using matplotlib.
+        """
+        if data_kws is None:
+            data_kws = {}
+        if fit_kws is None:
+            fit_kws = {}
+        if init_kws is None:
+            init_kws = {}
+        if ax_kws is None:
+            ax_kws = {}
+
+        if len(self.model.independent_vars) == 1:
+            independent_var = self.model.independent_vars[0]
+        else:
+            print('Fit can only be plotted if the model function has one '
+                  'independent variable.')
+            return False
+
+        if not isinstance(ax, plt.Axes):
+            ax = plt.gca(**ax_kws)
+
+        x_array = self.userkws[independent_var]
+
+        # make a dense array for x-axis if data is not dense
+        if numpoints is not None and len(self.data) < numpoints:
+            x_array_dense = np.linspace(x_array[0], x_array[-1], numpoints)
+        else:
+            x_array_dense = x_array
+
+        ax.plot(x_array_dense, self.model.eval(self.init_params,
+                **{independent_var: x_array_dense}), initfmt,
+                label='initial for ' + self.model.name, **init_kws)
+        ax.plot(x_array_dense, self.model.eval(self.params,
+                **{independent_var: x_array_dense}), fitfmt,
+                label=self.model.name, **fit_kws)
+
+        if self.weights is not None:
+            ax.errorbar(x_array, self.data, yerr=1/self.weights**0.5,
+                        fmt=datafmt, label='data', **data_kws)
+        else:
+            ax.plot(x_array, self.data, datafmt, label='data', **data_kws)
+
+        ax.set_xlabel(independent_var)
+        ax.set_ylabel('y')
+        ax.legend()
+
+        return ax
+
+    @_ensureMatplotlib
+    def plot_residuals(self, ax=None, datafmt='o', data_kws=None, fit_kws=None,
+                       ax_kws=None):
+        """Plot the fit residuals using matplotlib.
+
+        The method will plot residuals of the fit using matplotlib, including:
+        the data points and the fitted curve (as horizontal line). If the fit
+        model included weights, errorbars will also be plotted.
+
+        Parameters
+        ----------
+        ax : matplotlib.axes.Axes, optional
+            The axes to plot on. The default in None, which means use the
+            current pyplot axis or create one if there is none.
+        datafmt : string, optional
+            matplotlib format string for data points
+        data_kws : dictionary, optional
+            keyword arguments passed on to the plot function for data points
+        fit_kws : dictionary, optional
+            keyword arguments passed on to the plot function for fitted curve
+        ax_kws : dictionary, optional
+            keyword arguments for a new axis, if there is one being created
+
+        Returns
+        -------
+        matplotlib.axes.Axes
+
+        Notes
+        ----
+        For details about plot format strings and keyword arguments see
+        documentation of matplotlib.axes.Axes.plot.
+
+        If the fit model included weights, then matplotlib.axes.Axes.errorbar
+        is used to plot the data, with yerr set to 1 / self.weights**0.5
+
+        If `ax` is None then matplotlib.pyplot.gca(**ax_kws) is called.
+
+        See Also
+        --------
+        ModelFit.plot_fit : Plot the fit results using matplotlib.
+        ModelFit.plot : Plot the fit results and residuals using matplotlib.
+        """
+        if data_kws is None:
+            data_kws = {}
+        if fit_kws is None:
+            fit_kws = {}
+        if fit_kws is None:
+            fit_kws = {}
+        if ax_kws is None:
+            ax_kws = {}
+
+        if len(self.model.independent_vars) == 1:
+            independent_var = self.model.independent_vars[0]
+        else:
+            print('Fit can only be plotted if the model function has one '
+                  'independent variable.')
+            return False
+
+        if not isinstance(ax, plt.Axes):
+            ax = plt.gca(**ax_kws)
+
+        x_array = self.userkws[independent_var]
+
+        ax.axhline(0, label=self.model.name, **fit_kws)
+
+        if self.weights is not None:
+            ax.errorbar(x_array, self.eval() - self.data,
+                        yerr=1 / self.weights**0.5, fmt=datafmt,
+                        label='residuals', **data_kws)
+        else:
+            ax.plot(x_array, self.eval() - self.data, datafmt,
+                    label='residuals', **data_kws)
+
+        ax.set_ylabel('residuals')
+        ax.legend()
+
+        return ax
+
+    @_ensureMatplotlib
+    def plot(self, datafmt='o', fitfmt='-', initfmt='--', numpoints=None,
+             fig=None, data_kws=None, fit_kws=None, init_kws=None,
+             ax_res_kws=None, ax_fit_kws=None, fig_kws=None):
+        """Plot the fit results and residuals using matplotlib.
+
+        The method will produce a matplotlib figure with both results of the
+        fit and the residuals plotted. If the fit model included weights,
+        errorbars will also be plotted.
+
+        Parameters
+        ----------
+        datafmt : string, optional
+            matplotlib format string for data points
+        fitfmt : string, optional
+            matplotlib format string for fitted curve
+        initfmt : string, optional
+            matplotlib format string for initial conditions for the fit
+        numpoints : int, optional
+            If provided, the final and initial fit curves are evaluated not
+            only at data points, but refined to contain `numpoints` points in
+            total.
+        fig : matplotlib.figure.Figure, optional
+            The figure to plot on. The default in None, which means use the
+            current pyplot figure or create one if there is none.
+        data_kws : dictionary, optional
+            keyword arguments passed on to the plot function for data points
+        fit_kws : dictionary, optional
+            keyword arguments passed on to the plot function for fitted curve
+        init_kws : dictionary, optional
+            keyword arguments passed on to the plot function for the initial
+            conditions of the fit
+        ax_res_kws : dictionary, optional
+            keyword arguments for the axes for the residuals plot
+        ax_fit_kws : dictionary, optional
+            keyword arguments for the axes for the fit plot
+        fig_kws : dictionary, optional
+            keyword arguments for a new figure, if there is one being created
+
+        Returns
+        -------
+        matplotlib.figure.Figure
+
+        Notes
+        ----
+        The method combines ModelFit.plot_fit and ModelFit.plot_residuals.
+
+        If the fit model included weights, then matplotlib.axes.Axes.errorbar
+        is used to plot the data, with yerr set to 1 / self.weights**0.5
+
+        If `fig` is None then matplotlib.pyplot.figure(**fig_kws) is called.
+
+        See Also
+        --------
+        ModelFit.plot_fit : Plot the fit results using matplotlib.
+        ModelFit.plot_residuals : Plot the fit residuals using matplotlib.
+        """
+        if data_kws is None:
+            data_kws = {}
+        if fit_kws is None:
+            fit_kws = {}
+        if init_kws is None:
+            init_kws = {}
+        if ax_res_kws is None:
+            ax_res_kws = {}
+        if ax_fit_kws is None:
+            ax_fit_kws = {}
+        if fig_kws is None:
+            fig_kws = {}
+
+        if len(self.model.independent_vars) != 1:
+            print('Fit can only be plotted if the model function has one '
+                  'independent variable.')
+            return False
+
+        if not isinstance(fig, plt.Figure):
+            fig = plt.figure(**fig_kws)
+
+        gs = plt.GridSpec(nrows=2, ncols=1, height_ratios=[1, 4])
+        ax_res = fig.add_subplot(gs[0], **ax_res_kws)
+        ax_fit = fig.add_subplot(gs[1], sharex=ax_res, **ax_fit_kws)
+
+        self.plot_fit(ax=ax_fit, datafmt=datafmt, fitfmt=fitfmt,
+                      initfmt=initfmt, numpoints=numpoints, data_kws=data_kws,
+                      fit_kws=fit_kws, init_kws={}, ax_kws=ax_fit_kws)
+        self.plot_residuals(ax=ax_res, datafmt=datafmt, data_kws=data_kws,
+                            fit_kws=fit_kws, ax_kws=ax_res_kws)
+
+        return fig
