@@ -1,13 +1,13 @@
 from numpy import linspace, zeros, sin, exp, random, sqrt, pi, sign
 from lmfit import Parameters, Parameter, Minimizer
-from lmfit.utilfuncs import gaussian, loren, pvoigt
+from lmfit.lineshapes import gaussian, lorentzian, pvoigt
 from lmfit.printfuncs import report_fit
 
 def test_constraints1():
     def residual(pars, x, sigma=None, data=None):
         yg = gaussian(x, pars['amp_g'].value,
                       pars['cen_g'].value, pars['wid_g'].value)
-        yl = loren(x, pars['amp_l'].value,
+        yl = lorentzian(x, pars['amp_l'].value,
                    pars['cen_l'].value, pars['wid_l'].value)
 
         slope = pars['line_slope'].value
@@ -26,7 +26,7 @@ def test_constraints1():
     x = linspace(xmin, xmax, n)
 
     data = (gaussian(x, 21, 8.1, 1.2) +
-            loren(x, 10, 9.6, 2.4) +
+            lorentzian(x, 10, 9.6, 2.4) +
             random.normal(scale=0.23,  size=n) +
             x*0.5)
 
@@ -71,7 +71,7 @@ def test_constraints2():
     def residual(pars, x, sigma=None, data=None):
         yg = gaussian(x, pars['amp_g'].value,
                       pars['cen_g'].value, pars['wid_g'].value)
-        yl = loren(x, pars['amp_l'].value,
+        yl = lorentzian(x, pars['amp_l'].value,
                    pars['cen_l'].value, pars['wid_l'].value)
 
         slope = pars['line_slope'].value
@@ -90,7 +90,7 @@ def test_constraints2():
     x = linspace(xmin, xmax, n)
 
     data = (gaussian(x, 21, 8.1, 1.2) +
-            loren(x, 10, 9.6, 2.4) +
+            lorentzian(x, 10, 9.6, 2.4) +
             random.normal(scale=0.23,  size=n) +
             x*0.5)
 
@@ -103,7 +103,7 @@ def test_constraints2():
             Parameter(name='cen_l',  expr='1.5+cen_g'),
             Parameter(name='line_slope', value=0.0),
             Parameter(name='line_off', value=0.0)]
-    
+
     sigma = 0.021  # estimate of data error (for all data points)
 
     myfit = Minimizer(residual, pfit,
